:root {
    --default-color: dark green;
    --my-var: light red;
}

edit {
    color: var(--default-color);
}

#root {
    color: black;
    background: var(--my-var);
    background-adv: #515151;
}

linebox {
    color: white;
    background: black;
}

/*text {
    color: white;
    background: dark red;
}*/

.custom {
    color: light green;
    background: dark gray;
}

.dark {
    color: black;
    background: white;
}

#light {
    color: white;
    background: black;
}

#padded {
    color: white;
    background: black;
}

.flow {
    color: white;
    background: dark blue;
}

button {
    color: yellow;
}

button:focus {
    color: light red;
}

scrollbar,
/*text:focus:hover,*/
#texthover {
    color: light blue;
}

scrollbar:focus {
    color: black;
}

text.my_class {
    color: green;
}

#id:focus {
    color: light blue;
}

.col1 {
    background: dark blue;
}

.col2 {
    color: black;
    background: brown;
}

.inverted {
    color: black;
    background: white;
}
